/***************************************************************************
 *                                                                         *
 *   kemboss - Graphical user interface for EMBOSS                         *
 *                                                                         *
 *   Copyright (C) 2007-2019 Tobias Schwarz (mail@tobias-schwarz.net)      *
 *                                                                         *
 *   Based on the EMBOSS.kaptn scripts by Thomas Siegmund                  *
 *                                                                         *
 *   This program is free software; you can redistribute and/or modify     *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version, and provided that the above       *
 *   copyright and permission notice is included with all distributed      *
 *   copies of this or derived software.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 **************************************************************************/

#ifndef WINDOWCLASSES_H
#define WINDOWCLASSES_H

#include <QComboBox>
#include <QListWidget>
#include <QSpinBox>
#include <QtGlobal>
#include <QTemporaryDir>
#include <QtPrintSupport/QPrinter>

#include "kembossWindowClass.h"
#include "syntax_highlighter.h"
#include "runEmbossApp.h"

class FindDialog;
class FindReplaceDialog;

class kprogramWindow : public kembossWindow
// kprogramWindow is the abstract base class of all program windows
{
   Q_OBJECT

public:
   kprogramWindow(kembossWindow *parent = nullptr);
   virtual ~kprogramWindow() { }
   bool loadSettings(const QString &fileName);
   void loadFormats(const QString &fileName, QWidget *widget);
   void setCurrentFile(const QString &fileName);
   QString selectOpenFile(const QString &msg, QLineEdit *lineEdit);
   void selectAndOpenFile(const QString &msg, QLineEdit *lineEdit, QTextEdit *textEdit);
   QString selectSaveFile(const QString &msg, QLineEdit *lineEdit);
   void selectAndSaveFile(const QString &msg, QLineEdit *lineEdit, QTextEdit *textEdit);
   QStringList loadRebaseEnzymes();
   void saveEnzymeList(QListWidget *enzymeList = nullptr);
   QString selectedEnzymes(QListWidget *enzymeList = nullptr);
   QPointer<QComboBox> kpwProgramComboBox;
   QPointer<krunEmbossApp> runEmbossApp;
   void setTextEditFonts();
   void deleteTemporaryFiles();
   bool checkInputs();
   bool checkOutput();
   bool createProgramConfiguration();
   bool displayOutputFile();
   QPointer<QLineEdit> kpwOutputFileLineEdit;
   QPointer<QTextEdit> kpwOutputTextEdit;
   QPointer<QTabWidget> kpwMainTab;
   QStringList associatedQualifiers;
   QStringList additionalConfigurationParameters;

public slots:
   void fileRead(QTextEdit *textEdit, QString text);
   void finishedFileLoading(QTextEdit *textEdit);

   void openSAQForm(const QString num = "1");
   void openAssAQForm(const QString num = "1");
   void openOAQForm(const QString num = "2");
   void openOsAQForm(const QString num = "2");
   void openOfAQForm(const QString num = "3");
   void openFAQForm(const QString num = "1");
   void openGAQForm(const QString num = "");
   void openAAQForm(const QString num = "3");

private slots:
   void newSettings();
   void printDocument();
   void documentProperties();
   void find();
   void findReplace();

   void zoomIn();
   void zoomOut();
   void zoomOriginal();
   void rotateLeft();
   void rotateRight();

   void highlightSeq(bool highlight);

   void seqFormatNone();
   void seqFormatFastaAA();
   void seqFormatFastaNuc();
   void seqFormatAlignmentAA();
   void seqFormatAlignmentNuc();
   void seqFormatEmbl();
   void seqFormatGbAA();
   void seqFormatGbNuc();
   void seqFormatSwiss();
   void seqFormatChanges(int index);

   void selectionChanged();
   void embossHelp();
   bool runEmboss();
   void embossExecutionFinished(const int exitCode);
   bool on_OpenInputFilePushButton_clicked();
   bool on_OpenInputFilePushButton2_clicked();
   bool on_SaveInputFileAsPushButton_clicked();
   bool on_SaveInputFileAsPushButton2_clicked();
   bool on_SaveOutputFileAsPushButton_clicked();
   void tabChanged(const int &index);
   //void viewTab();
   void wasModified() { setWindowModified(true); saveAct->setEnabled(true); }
   void copyCommandline();
   void addParameter();

private:
   void createActions();
   void createMenus();
   void createToolbars();
   void findControls();
   void setTextEditFont(QTextEdit *textEdit);
   bool saveSettings(const QString &fileName);
   bool checkInput(QTextEdit *inputTextEdit, QLineEdit *inputFileLineEdit);
   bool maybeSave();
   bool openInputFilePushButtonClicked(QTextEdit *inputTextEdit, QLineEdit *inputFileLineEdit);

   bool maybeSaveInputFiles();
   bool maybeSaveInputFile(QTextEdit *inputTextEdit, QLineEdit *inputFileLineEdit);
   bool saveInputFileAsPushButtonClicked(QTextEdit *inputTextEdit, QLineEdit *inputFileLineEdit);
   bool saveTextToTempfile(QTextEdit *textEdit, QLineEdit *lineEdit);
   bool maybeSaveOutputFile();
   bool saveOutputFile(const QString &fileName);

   void loadPrinterSettings();
   void savePrinterSettings();
   void loadOutputGraphics(const QString &filename);
   void selectSeqFormat(const QString &fileName);
   bool charIsAminoacid(const QChar &seq);
   bool charIsNucleotide(const QChar &seq);
   bool sequenceIsNucleotide(const QString &seq);
   // Returns true if the word characters of the first one hundred characters of the string seq consist of at least 90% A, T, G, C or N
   QStringList tempInputFiles;
   QTemporaryDir *tempDir;
   QString tempOutputFile;
   QString defaultSettingsFile();
   QPrinter printer;
   qreal graphScale;

   QMenu *fileMenu;
   QMenu *editMenu;
   QMenu *graphMenu;
   QMenu *viewMenu;
   QMenu *seqFormatMenu;
   QMenu *toolbarsMenu;
   //QMenu *tabsMenu;
   QMenu *runMenu;
   QMenu *helpMenu;

   QAction *newAct;
   QAction *openAct;
   QAction *saveAct;
   QAction *saveAsAct;
   QAction *exitAct;
   QAction *runAct;
   QAction *addParameterAct;
   QAction *copyCommandlineAct;
   QAction *cutAct;
   QAction *copyAct;
   QAction *pasteAct;
   QAction *undoAct;
   QAction *redoAct;
   QAction *findAct;
   QAction *findNextAct;
   QAction *findReplaceAct;
   QAction *helpAct;
   QAction *whatsThisAct;
   QAction *printAct;
   QAction *propertiesAct;
   QAction *zoomInAct;
   QAction *zoomOutAct;
   QAction *zoomOrigAct;
   QAction *rotateLeftAct;
   QAction *rotateRightAct;

   QAction *highlightSeqAct;
   QAction *seqFormatNoneAct;
   QAction *seqFormatFastaAAAct;
   QAction *seqFormatFastaNucAct;
   QAction *seqFormatAlignmentAAAct;
   QAction *seqFormatAlignmentNucAct;
   QAction *seqFormatEmblAct;
   QAction *seqFormatGbAAAct;
   QAction *seqFormatGbNucAct;
   QAction *seqFormatSwissAct;

   QPointer<QTextEdit> kpwInputTextEdit;
   QPointer<QLineEdit> kpwInputFileLineEdit;   
   QPointer<QTextEdit> kpwInputTextEdit2;
   QPointer<QLineEdit> kpwInputFileLineEdit2;
   QPointer<QTextEdit> currentTextEdit;

   QPointer<QSpinBox> selectFromSpinBox;
   QPointer<QSpinBox> selectToSpinBox;

   QPointer<Highlighter> highlighter;

   QPointer<QComboBox> selectSeqFormatComboBox;

   QPointer<FindDialog> m_findDialog;
   QPointer<FindReplaceDialog> m_findReplaceDialog;

   QPointer<kembossSettings> ksettings;

protected:
   bool loadInputFile(QTextEdit *inputTextEdit, const QString &fileName);
   bool saveInputFile(QTextEdit *inputTextEdit, const QString &fileName);
   void postWindowCreated();
   bool eventFilter(QObject *object, QEvent *event);
   void closeEvent(QCloseEvent *event);
   virtual void beforeRunningProgram() { }
   //virtual void afterRunningProgram();
   void completeOutputFilename(QLineEdit *lineEdit, const QString &extension);
   QStringList inputFiles;
   QStringList outputFiles;
};

#endif
